==> 0.0.9.tar.gz: android/src/main/kotlin/com/app/spectus_payumoney_plugin/SpectusPayumoneyPlugin.kt <==
package com.app.spectus_payumoney_plugin

import android.R
import android.app.Activity
import android.content.Context
import android.content.Intent
import android.util.Log
import androidx.annotation.NonNull
import com.app.spectus_payumoney_plugin.payumoney.AppEnvironment
import com.app.spectus_payumoney_plugin.payumoney.AppPreference
import com.payumoney.core.PayUmoneyConfig
import com.payumoney.core.PayUmoneyConstants
import com.payumoney.core.PayUmoneySdkInitializer
import com.payumoney.core.entity.TransactionResponse
import com.payumoney.sdkui.ui.utils.PayUmoneyFlowManager
import com.payumoney.sdkui.ui.utils.ResultModel

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import org.json.JSONObject
import java.security.MessageDigest
import java.security.NoSuchAlgorithmException

/** SpectusPayumoneyPlugin */
class SpectusPayumoneyPlugin: FlutterPlugin, MethodChannel.MethodCallHandler, ActivityAware, io.flutter.plugin.common.PluginRegistry.ActivityResultListener  {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var channel : MethodChannel

  val TAG = "MainActivity : "
  var successResult : Result? = null
  var mActivity : Activity? = null
  var mContext : Context? = null
  var email : String?  = ""
  var amount : String? = ""
  var mobile : String?= ""
  var name : String?= ""
  var merchantID : String?= ""
  var merchantKey : String?= ""
  var salt : String?= ""
  var environmentMode : Int? = 0 // 0- SANDBOX && 1 - LIVE MODE

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "spectus_payumoney_plugin")
    mContext = flutterPluginBinding.applicationContext
    channel.setMethodCallHandler(this)
  }

  companion object {
    @JvmStatic
    fun registerWith(registrar: Registrar) {
      val channel = MethodChannel(registrar.messenger(), "spectus_payumoney_plugin")
      channel.setMethodCallHandler(SpectusPayumoneyPlugin())
    }
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    if (call.method == "getPlatformVersion") {
      successResult = result
      result.success("Android ${android.os.Build.VERSION.RELEASE}")
    }
    else if (call.method == "payUmoneyPayment")
    {
      successResult = result
      Log.e("Method Called","payUmoneyPayment")
      merchantID = call.argument<String?>("merchantID")
      merchantKey = call.argument<String?>("merchantKey")
      salt = call.argument<String?>("salt")
      email = call.argument<String?>("email")
      mobile = call.argument<String?>("mobile")
      amount = call.argument<String?>("amount")
      name = call.argument<String?>("name")
      environmentMode = call.argument<Int?>("environmentMode")
      Log.e("environmentMode",""+environmentMode)
      launchPayUMoneyFlow()
    }
    else {
      result.notImplemented()
    }
  }

  private fun launchPayUMoneyFlow() {

    val payUmoneyConfig = PayUmoneyConfig.getInstance()

    //Use this to set your custom text on result screen button
    payUmoneyConfig.doneButtonText = "Done"

    //Use this to set your custom title for the activity
    payUmoneyConfig.payUmoneyActivityTitle = name

    payUmoneyConfig.disableExitConfirmation(false)

    val builder = PayUmoneySdkInitializer.PaymentParam.Builder()

    /*var amount = 0.0
    try {
      amount = java.lang.Double.parseDouble(amount)

    } catch (e: Exception) {
      e.printStackTrace()
    }*/

    val txnId = System.currentTimeMillis().toString() + ""
    //String txnId = "TXNID720431525261327973";
    val phone = mobile
    val productName = name
    val firstName = name
    val email = email
    //val udf1 = "Charged by Gujarat State Pharmacy Council"
    val udf1 = ""
    val udf2 = "Charged amount:"+" "+amount
    val udf3 = ""
    val udf4 = ""
    val udf5 = ""
    val udf6 = ""
    val udf7 = ""
    val udf8 = ""
    val udf9 = ""
    val udf10 =
    var  mPaymentParams : PayUmoneySdkInitializer.PaymentParam


    var appEnvironment : AppEnvironment? = null

    if(environmentMode == 0){
      appEnvironment = AppEnvironment.SANDBOX
    }else{
      appEnvironment = AppEnvironment.PRODUCTION
    }


//    if((mContext as? PayUMoneyApp)?.appEnvironment !=null){
//      appEnvironment = (mContext as? PayUMoneyApp)?.appEnvironment
//    }else{
//      Log.e("appEnvironment","Called")
//    }


    builder.setAmount(amount.toString())
            .setTxnId(txnId)
            .setPhone(phone)
            .setProductName(productName)
            .setFirstName(firstName)
            .setEmail(email)
            .setsUrl(appEnvironment?.surl())
            .setfUrl(appEnvironment?.furl())
            .setUdf1(udf1)
            .setUdf2(udf2)
            .setUdf3(udf3)
            .setUdf4(udf4)
            .setUdf5(udf5)
            .setUdf6(udf6)
            .setUdf7(udf7)
            .setUdf8(udf8)
            .setUdf9(udf9)
            .setUdf10(udf10)
            .setIsDebug(appEnvironment?.debug())
            .setKey(merchantKey)
            .setMerchantId(merchantID)

            /*.setIsDebug(appEnvironment?.debug())
            .setKey(appEnvironment?.merchant_Key())
            .setMerchantId(appEnvironment?.merchant_ID())*/

    try {
      mPaymentParams = builder.build()

      /*
             * Hash should always be generated from your server side.
             * */
//        generateHashFromServer(mPaymentParams);

      /*            */
      /**
       * Do not use below code when going live
       * Below code is provided to generate hash from sdk.
       * It is recommended to generate hash from server side only.
       */
      mPaymentParams = calculateServerSideHashAndInitiatePayment1(mPaymentParams)

      if(mActivity==null){
        Log.e("Mactivity","Null")
      }

      if (AppPreference.selectedTheme != -1) {
        PayUmoneyFlowManager.startPayUMoneyFlow(mPaymentParams, mActivity!!, AppPreference.selectedTheme,false)
      } else {
        PayUmoneyFlowManager.startPayUMoneyFlow(mPaymentParams, mActivity!!, R.style.Theme, false)
      }

    } catch (e: Exception) {
      // some exception occurred
      Log.e("Exception",e.message);
//      Toast.makeText(mActivity, e.message, Toast.LENGTH_LONG).show()
    }

  }

  private fun calculateServerSideHashAndInitiatePayment1(paymentParam: PayUmoneySdkInitializer.PaymentParam): PayUmoneySdkInitializer.PaymentParam {

    val stringBuilder = StringBuilder()
    val params = paymentParam.params
    stringBuilder.append(params[PayUmoneyConstants.KEY] + "|")
    stringBuilder.append(params[PayUmoneyConstants.TXNID] + "|")
    stringBuilder.append(params[PayUmoneyConstants.AMOUNT] + "|")
    stringBuilder.append(params[PayUmoneyConstants.PRODUCT_INFO] + "|")
    stringBuilder.append(params[PayUmoneyConstants.FIRSTNAME] + "|")
    stringBuilder.append(params[PayUmoneyConstants.EMAIL] + "|")
    stringBuilder.append(params[PayUmoneyConstants.UDF1] + "|")
    stringBuilder.append(params[PayUmoneyConstants.UDF2] + "|")
    stringBuilder.append(params[PayUmoneyConstants.UDF3] + "|")
    stringBuilder.append(params[PayUmoneyConstants.UDF4] + "|")
    stringBuilder.append(params[PayUmoneyConstants.UDF5] + "|||||"+salt)

    var appEnvironment : AppEnvironment? = null

    if(environmentMode == 0){
      appEnvironment = AppEnvironment.SANDBOX
    }else{
      appEnvironment = AppEnvironment.PRODUCTION
    }

    stringBuilder.append(appEnvironment?.salt())


    val hash = hashCal(stringBuilder.toString())
    Log.e("hash",""+hash)
    paymentParam.setMerchantHash(hash)

    Log.e("Calculate","Method Called")
    return paymentParam
  }

  fun hashCal(str: String): String {
    val hashseq = str.toByteArray()
    val hexString = StringBuilder()
    try {
      val algorithm = MessageDigest.getInstance("SHA-512")
      algorithm.reset()
      algorithm.update(hashseq)
      val messageDigest = algorithm.digest()
      for (aMessageDigest in messageDigest) {
        val hex = Integer.toHexString(0xFF and aMessageDigest.toInt())
        if (hex.length == 1) {
          hexString.append("0")
        }
        hexString.append(hex)
      }
    } catch (ignored: NoSuchAlgorithmException) {
    }

    return hexString.toString()
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }

  override fun onDetachedFromActivity() {
    //TODO("Not yet implemented")
  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    //TODO("Not yet implemented")
    onAttachedToActivity(binding)
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    //TODO("Not yet implemented")
    Log.e("onAttachedToActivity","Called")
    if(binding.activity == null){
      Log.e("binding.activity","null")
    }
    mActivity = binding.activity
    binding.addActivityResultListener(this)
  }

  override fun onDetachedFromActivityForConfigChanges() {
    //TODO("Not yet implemented")
  }

  override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?): Boolean {
    //TODO("Not yet implemented")
    Log.d("MainActivity", "request code " + requestCode + " resultcode " + resultCode);
    if (requestCode == PayUmoneyFlowManager.REQUEST_CODE_PAYMENT && resultCode == Activity.RESULT_OK && data != null)
    {

      var transactionResponse : TransactionResponse?  = null
      var resultModel : ResultModel?  = null
      var payuResponse : String?  = ""
      var merchantResponse : String?  = ""

      transactionResponse = data.getParcelableExtra (PayUmoneyFlowManager.INTENT_EXTRA_TRANSACTION_RESPONSE);
      resultModel = data.getParcelableExtra (PayUmoneyFlowManager.ARG_RESULT);

      // Check which object is non-null
      if (transactionResponse != null && transactionResponse.getPayuResponse() != null)
      {
        if (transactionResponse.getTransactionStatus().equals(TransactionResponse.TransactionStatus.SUCCESSFUL))
        {
          //Success Transaction
          // Response from Payumoney
          payuResponse = transactionResponse.getPayuResponse()
          Log.e("success response",""+payuResponse)

          val emp = JSONObject(payuResponse).optJSONObject("result")
          val paymentId = emp.getString("paymentId")
          Log.e("paymentId",""+paymentId)
          successResult!!.success(paymentId)

          // Response from SURl and FURL
          merchantResponse = transactionResponse . getTransactionDetails ()
          return true
        } else {
          //Failure Transaction
          return false
        }
      } else if (resultModel != null && resultModel.getError() != null) {
        Log.d(TAG, "Error response : " + resultModel.getError().getTransactionResponse());
      } else {
        Log.d(TAG, "Both objects are null!");
      }
    }
    return  false
  }
}

==> 0.0.11.tar.gz: android/src/main/kotlin/com/app/spectus_payumoney_plugin/SpectusPayumoneyPlugin.kt <==
package com.app.spectus_payumoney_plugin

import android.R
import android.app.Activity
import android.content.Context
import android.content.Intent
import android.util.Log
import androidx.annotation.NonNull
import com.app.spectus_payumoney_plugin.payumoney.AppEnvironment
import com.app.spectus_payumoney_plugin.payumoney.AppPreference
import com.payumoney.core.PayUmoneyConfig
import com.payumoney.core.PayUmoneyConstants
import com.payumoney.core.PayUmoneySdkInitializer
import com.payumoney.core.entity.TransactionResponse
import com.payumoney.sdkui.ui.utils.PayUmoneyFlowManager
import com.payumoney.sdkui.ui.utils.ResultModel

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import org.json.JSONObject
import java.security.MessageDigest
import java.security.NoSuchAlgorithmException

/** SpectusPayumoneyPlugin */
class SpectusPayumoneyPlugin: FlutterPlugin, MethodChannel.MethodCallHandler, ActivityAware, io.flutter.plugin.common.PluginRegistry.ActivityResultListener  {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var channel : MethodChannel

  val TAG = "MainActivity : "
  var successResult : Result? = null
  var mActivity : Activity? = null
  var mContext : Context? = null
  var email : String?  = ""
  var amount : String? = ""
  var mobile : String?= ""
  var name : String?= ""
  var merchantID : String?= ""
  var merchantKey : String?= ""
  var salt : String?= ""
  var environmentMode : Int? = 0 // 0- SANDBOX && 1 - LIVE MODE

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "spectus_payumoney_plugin")
    mContext = flutterPluginBinding.applicationContext
    channel.setMethodCallHandler(this)
  }

  companion object {
    @JvmStatic
    fun registerWith(registrar: Registrar) {
      val channel = MethodChannel(registrar.messenger(), "spectus_payumoney_plugin")
      channel.setMethodCallHandler(SpectusPayumoneyPlugin())
    }
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    if (call.method == "getPlatformVersion") {
      successResult = result
      result.success("Android ${android.os.Build.VERSION.RELEASE}")
    }
    else if (call.method == "payUmoneyPayment")
    {
      successResult = result
      Log.e("Method Called","payUmoneyPayment")
      merchantID = call.argument<String?>("merchantID")
      merchantKey = call.argument<String?>("merchantKey")
      salt = call.argument<String?>("salt")
      email = call.argument<String?>("email")
      mobile = call.argument<String?>("mobile")
      amount = call.argument<String?>("amount")
      name = call.argument<String?>("name")
      environmentMode = call.argument<Int?>("environmentMode")
      Log.e("environmentMode",""+environmentMode)
      launchPayUMoneyFlow()
    }
    else {
      result.notImplemented()
    }
  }

  private fun launchPayUMoneyFlow() {

    val payUmoneyConfig = PayUmoneyConfig.getInstance()

    //Use this to set your custom text on result screen button
    payUmoneyConfig.doneButtonText = "Done"

    //Use this to set your custom title for the activity
    payUmoneyConfig.payUmoneyActivityTitle = name

    payUmoneyConfig.disableExitConfirmation(false)

    val builder = PayUmoneySdkInitializer.PaymentParam.Builder()

    /*var amount = 0.0
    try {
      amount = java.lang.Double.parseDouble(amount)

    } catch (e: Exception) {
      e.printStackTrace()
    }*/

    val txnId = System.currentTimeMillis().toString() + ""
    //String txnId = "TXNID720431525261327973";
    val phone = mobile
    val productName = name
    val firstName = name
    val email = email
    //val udf1 = "Charged by Gujarat State Pharmacy Council"
    val udf1 = ""
    val udf2 = "Charged amount:"+" "+amount
    val udf3 = ""
    val udf4 = ""
    val udf5 = ""
    val udf6 = ""
    val udf7 = ""
    val udf8 = ""
    val udf9 = ""
    val udf10 = ""
    var  mPaymentParams : PayUmoneySdkInitializer.PaymentParam


    var appEnvironment : AppEnvironment? = null

    if(environmentMode == 0){
      appEnvironment = AppEnvironment.SANDBOX
    }else{
      appEnvironment = AppEnvironment.PRODUCTION
    }


//    if((mContext as? PayUMoneyApp)?.appEnvironment !=null){
//      appEnvironment = (mContext as? PayUMoneyApp)?.appEnvironment
//    }else{
//      Log.e("appEnvironment","Called")
//    }


    builder.setAmount(amount.toString())
            .setTxnId(txnId)
            .setPhone(phone)
            .setProductName(productName)
            .setFirstName(firstName)
            .setEmail(email)
            .setsUrl(appEnvironment?.surl())
            .setfUrl(appEnvironment?.furl())
            .setUdf1(udf1)
            .setUdf2(udf2)
            .setUdf3(udf3)
            .setUdf4(udf4)
            .setUdf5(udf5)
            .setUdf6(udf6)
            .setUdf7(udf7)
            .setUdf8(udf8)
            .setUdf9(udf9)
            .setUdf10(udf10)
            .setIsDebug(appEnvironment?.debug())
            .setKey(merchantKey)
            .setMerchantId(merchantID)

            /*.setIsDebug(appEnvironment?.debug())
            .setKey(appEnvironment?.merchant_Key())
            .setMerchantId(appEnvironment?.merchant_ID())*/

    try {
      mPaymentParams = builder.build()

      /*
             * Hash should always be generated from your server side.
             * */
//        generateHashFromServer(mPaymentParams);

      /*            */
      /**
       * Do not use below code when going live
       * Below code is provided to generate hash from sdk.
       * It is recommended to generate hash from server side only.
       */
      mPaymentParams = calculateServerSideHashAndInitiatePayment1(mPaymentParams)

      if(mActivity==null){
        Log.e("Mactivity","Null")
      }

      if (AppPreference.selectedTheme != -1) {
        PayUmoneyFlowManager.startPayUMoneyFlow(mPaymentParams, mActivity!!, AppPreference.selectedTheme,false)
      } else {
        PayUmoneyFlowManager.startPayUMoneyFlow(mPaymentParams, mActivity!!, R.style.Theme, false)
      }

    } catch (e: Exception) {
      // some exception occurred
      Log.e("Exception",e.message);
//      Toast.makeText(mActivity, e.message, Toast.LENGTH_LONG).show()
    }

  }

  private fun calculateServerSideHashAndInitiatePayment1(paymentParam: PayUmoneySdkInitializer.PaymentParam): PayUmoneySdkInitializer.PaymentParam {

    val stringBuilder = StringBuilder()
    val params = paymentParam.params
    stringBuilder.append(params[PayUmoneyConstants.KEY] + "|")
    stringBuilder.append(params[PayUmoneyConstants.TXNID] + "|")
    stringBuilder.append(params[PayUmoneyConstants.AMOUNT] + "|")
    stringBuilder.append(params[PayUmoneyConstants.PRODUCT_INFO] + "|")
    stringBuilder.append(params[PayUmoneyConstants.FIRSTNAME] + "|")
    stringBuilder.append(params[PayUmoneyConstants.EMAIL] + "|")
    stringBuilder.append(params[PayUmoneyConstants.UDF1] + "|")
    stringBuilder.append(params[PayUmoneyConstants.UDF2] + "|")
    stringBuilder.append(params[PayUmoneyConstants.UDF3] + "|")
    stringBuilder.append(params[PayUmoneyConstants.UDF4] + "|")
    stringBuilder.append(params[PayUmoneyConstants.UDF5] + "||||||")

    var appEnvironment : AppEnvironment? = null

    if(environmentMode == 0){
      appEnvironment = AppEnvironment.SANDBOX
    }else{
      appEnvironment = AppEnvironment.PRODUCTION
    }

   // stringBuilder.append(appEnvironment?.salt())
    stringBuilder.append(salt)


    val hash = hashCal(stringBuilder.toString())
    Log.e("hash",""+hash)
    paymentParam.setMerchantHash(hash)

    Log.e("Calculate","Method Called")
    return paymentParam
  }

  fun hashCal(str: String): String {
    val hashseq = str.toByteArray()
    val hexString = StringBuilder()
    try {
      val algorithm = MessageDigest.getInstance("SHA-512")
      algorithm.reset()
      algorithm.update(hashseq)
      val messageDigest = algorithm.digest()
      for (aMessageDigest in messageDigest) {
        val hex = Integer.toHexString(0xFF and aMessageDigest.toInt())
        if (hex.length == 1) {
          hexString.append("0")
        }
        hexString.append(hex)
      }
    } catch (ignored: NoSuchAlgorithmException) {
    }

    return hexString.toString()
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }

  override fun onDetachedFromActivity() {
    //TODO("Not yet implemented")
  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    //TODO("Not yet implemented")
    onAttachedToActivity(binding)
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    //TODO("Not yet implemented")
    Log.e("onAttachedToActivity","Called")
    if(binding.activity == null){
      Log.e("binding.activity","null")
    }
    mActivity = binding.activity
    binding.addActivityResultListener(this)
  }

  override fun onDetachedFromActivityForConfigChanges() {
    //TODO("Not yet implemented")
  }

  override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?): Boolean {
    //TODO("Not yet implemented")
    Log.d("MainActivity", "request code " + requestCode + " resultcode " + resultCode);
    if (requestCode == PayUmoneyFlowManager.REQUEST_CODE_PAYMENT && resultCode == Activity.RESULT_OK && data != null)
    {

      var transactionResponse : TransactionResponse?  = null
      var resultModel : ResultModel?  = null
      var payuResponse : String?  = ""
      var merchantResponse : String?  = ""

      transactionResponse = data.getParcelableExtra (PayUmoneyFlowManager.INTENT_EXTRA_TRANSACTION_RESPONSE);
      resultModel = data.getParcelableExtra (PayUmoneyFlowManager.ARG_RESULT);

      // Check which object is non-null
      if (transactionResponse != null && transactionResponse.getPayuResponse() != null)
      {
        if (transactionResponse.getTransactionStatus().equals(TransactionResponse.TransactionStatus.SUCCESSFUL))
        {
          //Success Transaction
          // Response from Payumoney
          payuResponse = transactionResponse.getPayuResponse()
          Log.e("success response",""+payuResponse)

          val emp = JSONObject(payuResponse).optJSONObject("result")
          val paymentId = emp.getString("paymentId")
          Log.e("paymentId",""+paymentId)
          successResult!!.success(paymentId)

          // Response from SURl and FURL
          merchantResponse = transactionResponse . getTransactionDetails ()
          return true
        } else {
          //Failure Transaction
          return false
        }
      } else if (resultModel != null && resultModel.getError() != null) {
        Log.d(TAG, "Error response : " + resultModel.getError().getTransactionResponse());
      } else {
        Log.d(TAG, "Both objects are null!");
      }
    }
    return  false
  }
}
